﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using WindowsFormsApp2.Properties;
using GrapeCity.Spreadsheet;
using FarPoint.Win.Spread;

namespace WindowsFormsApp2
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            using (System.IO.MemoryStream s = new System.IO.MemoryStream(Resources.SudokuSolver))
            {
                fpSpread1.OpenExcel(s);
                // fix up columns
                IWorkbook wb = fpSpread1.AsWorkbook();
                IWorksheet ws = wb.ActiveSheet;
                // intialize a puzzle
                const object O = null;
                ws.SetValue(4, 1, new object[,] { { 3, O, 2, O, 7, O, 5, O, 4 },
                                                  { O, O, O, O, O, O, O, O, O },
                                                  { O, O, 6, 2, 5, 4, O, O, O },
                                                  { O, 9, O, O, O, O, 6, O, O },
                                                  { O, O, 8, 9, 6, 5, 1, O, O },
                                                  { O, O, 5, O, O, O, O, 2, O },
                                                  { O, O, O, 1, 9, 7, 8, O, O },
                                                  { O, O, O, O, O, O, O, O, O },
                                                  { 5, O, 7, O, 8, O, 2, O, 6 } });
                ws.Columns[1].AutoFit();
                ws.Columns[22].AutoFit();
                ws.Columns[30].AutoFit();
                ws.Columns[39].AutoFit();
                ws.Columns[48].AutoFit();
                ws.Columns[52].AutoFit();
                // unprotect sheet so formulas can be edited
                ws.Unprotect(null);
                // setup iterations and calculation hokey
                wb.WorkbookSet.CalculationEngine.Iterative = true;
                wb.WorkbookSet.CalculationEngine.MaximumIterations = 1;
                fpSpread1.GetActionMap().Put("Recalculate", new RecalculateAction());
                fpSpread1.GetInputMap(InputMapMode.WhenFocused).Put(new Keystroke(Keys.F9, Keys.None), "Recalculate");
            }
        }

       private void OpenToolStripMenuItem_Click(object sender, EventArgs e)
       {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "Excel Spreadsheet (*.xlsx;*.xlsm;*.xls)|*.xlsx;*.xlsm;*.xls|All Files (*.*)|*.*";
            ofd.FilterIndex = 0;
            ofd.Title = "Open Spreadsheet";
            if (DialogResult.OK == ofd.ShowDialog())
            {
                fpSpread1.Open(ofd.FileName);
                // unprotect sheets so formulas can be edited
                foreach ( SheetView sv in fpSpread1.Sheets)
                    sv.Protect = false;
            }
        }

        private void ExitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void ShowDependentsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            fpSpread1.ShowDependents(fpSpread1.ActiveSheetIndex, fpSpread1.ActiveSheet.ActiveRowIndex, fpSpread1.ActiveSheet.ActiveColumnIndex);
        }

        private void ShowPrecedentsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            fpSpread1.ShowPrecedents(fpSpread1.ActiveSheetIndex, fpSpread1.ActiveSheet.ActiveRowIndex, fpSpread1.ActiveSheet.ActiveColumnIndex);
        }

        private void RemoveArrowsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < fpSpread1.Sheets.Count; i++)
            {
                fpSpread1.HideDependents(i);
                fpSpread1.HidePrecedents(i);
            }
        }

        private void GenerateReportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            int row, rowStart;
            IWorkbook wb = fpSpread1.AsWorkbook();
            IWorkbook wb2 = fpSpread2.AsWorkbook();
            IWorksheet wks = wb2.ActiveSheet;
            IRange targetCell = wb.ActiveSheet.Cells[fpSpread1.ActiveSheet.ActiveRowIndex, fpSpread1.ActiveSheet.ActiveColumnIndex];
            if (!(bool)targetCell.HasFormula)
            {
                MessageBox.Show("You must select a cell containing a formula to analyze first!");
                return;
            }
            IAreas direct_dependents = targetCell.DirectDependents.Areas;
            IAreas direct_precedents = targetCell.DirectPrecedents.Areas;
            IAreas all_dependents = targetCell.Dependents.Areas;
            IAreas all_precedents = targetCell.Precedents.Areas;
            SheetView sv = fpSpread2.ActiveSheet;
            sv.Reset();
            sv.RowCount = int.MaxValue;
            sv.Cells[0, 0].Text = "Tracing Formula in cell " + targetCell.Address(false, false, ReferenceStyle.A1, false, targetCell) + ":";
            wks.Cells[0, 0].ApplyStyle(BuiltInStyle.Percent60Accent6);
            wks.Cells[0, 0].ApplyStyle(BuiltInStyle.Title);
            wks.Rows[0].AutoFit();
            sv.Cells[1, 0].Text = targetCell.HasArray ? targetCell.FormulaArray : targetCell.Formula;
            sv.Cells[1, 0].ColumnSpan = 2;
            wks.Cells[1, 0].Style.VerticalAlignment = VerticalAlignment.Top;
            wks.Cells[1, 0].Style.WrapText = true;
            sv.SetRowHeight(1, sv.GetPreferredCellSize(1, 0).Height);
            wks.Cells[1, 0].ApplyStyle(BuiltInStyle.Percent20Accent6);
            wks.Cells[1, 0].ApplyStyle(BuiltInStyle.Calculation);

            row = 3;
            sv.Cells[row, 0].Text = "Direct Dependents";
            sv.Cells[row, 0].Note = "Direct Depedents are cells which directly reference the target cell.";
            sv.Cells[row, 0].ColumnSpan = 2;
            wks.Cells[row, 0].ApplyStyle(BuiltInStyle.Percent40Accent6);
            wks.Cells[row, 0].ApplyStyle(BuiltInStyle.Title);
            wks.Rows[row].RowHeight = wks.Rows[0].RowHeight;
            row++;
            rowStart = row;
            sv.Cells[row, 0].Text = "Cell";
            sv.Cells[row, 1].Text = "Formula/Value";
            row++;
            for (int i = 0; i < direct_dependents.Count; row++, i++ )
            {
                sv.Cells[row, 0].Text = direct_dependents[i].Address(false, false, ReferenceStyle.A1, false, direct_dependents[i]);
                sv.Cells[row, 1].Text = (bool)direct_dependents[i].HasFormula ? (direct_dependents[i].HasArray ? direct_dependents[i].FormulaArray : direct_dependents[i].Formula) : direct_dependents[i].Text;
            }
            sv.AddTable(rowStart, 0, row - rowStart, 2);
            for (int i = 0; i < direct_dependents.Count; i++)
            {
                if ((bool)direct_dependents[i].HasFormula)
                {
                    wks.Cells[rowStart + 1 + i, 1].ApplyStyle(BuiltInStyle.Calculation);
                    wks.Cells[rowStart + 1 + i, 1].Style.WrapText = true;
                    sv.SetRowHeight(rowStart + 1 + i, sv.GetPreferredCellSize(rowStart + 1 + i, 1).Height);
                }
            }
            row += 2;
            sv.Cells[row, 0].Text = "Direct Precedents";
            sv.Cells[row, 0].Note = "Direct Precedents are cells which are directly referenced in the target cell formula.";
            sv.Cells[row, 0].ColumnSpan = 2;
            wks.Cells[row, 0].ApplyStyle(BuiltInStyle.Percent40Accent6);
            wks.Cells[row, 0].ApplyStyle(BuiltInStyle.Title);
            wks.Rows[row].RowHeight = wks.Rows[0].RowHeight;
            row++;
            rowStart = row;
            sv.Cells[row, 0].Text = "Cell";
            sv.Cells[row, 1].Text = "Formula/Value";
            row++;
            for (int i = 0; i < direct_precedents.Count; row++, i++)
            {
                sv.Cells[row, 0].Text = direct_precedents[i].Address(false, false, ReferenceStyle.A1, false, direct_precedents[i]);
                sv.Cells[row, 1].Text = (bool)direct_precedents[i].HasFormula ? (direct_precedents[i].HasArray ? direct_precedents[i].FormulaArray : direct_precedents[i].Formula) : direct_precedents[i].Text;
            }
            sv.AddTable(rowStart, 0, row - rowStart, 2);
            for (int i = 0; i < direct_precedents.Count; i++)
            {
                if ((bool)direct_precedents[i].HasFormula)
                {
                    wks.Cells[rowStart + 1 + i, 1].ApplyStyle(BuiltInStyle.Calculation);
                    wks.Cells[rowStart + 1 + i, 1].Style.WrapText = true;
                    sv.SetRowHeight(rowStart + 1 + i, sv.GetPreferredCellSize(rowStart + 1 + i, 1).Height);
                }
            }
            if (all_dependents.Count > direct_dependents.Count)
            {
                row += 2;
                sv.Cells[row, 0].Text = "All Dependents";
                sv.Cells[row, 0].Note = "Depedents are cells which directly reference the target cell, and all cells which directly reference those cells, and so on.";
                sv.Cells[row, 0].ColumnSpan = 2;
                wks.Cells[row, 0].ApplyStyle(BuiltInStyle.Percent40Accent6);
                wks.Cells[row, 0].ApplyStyle(BuiltInStyle.Title);
                wks.Rows[row].RowHeight = wks.Rows[0].RowHeight;
                row++;
                rowStart = row;
                sv.Cells[row, 0].Text = "Cell";
                sv.Cells[row, 1].Text = "Formula/Value";
                row++;
                for (int i = 0; i < all_dependents.Count; row++, i++)
                {
                    sv.Cells[row, 0].Text = all_dependents[i].Address(false, false, ReferenceStyle.A1, false, all_dependents[i]);
                    sv.Cells[row, 1].Text = (bool)all_dependents[i].HasFormula ? (all_dependents[i].HasArray ? all_dependents[i].FormulaArray : all_dependents[i].Formula) : all_dependents[i].Text;
                }
                sv.AddTable(rowStart, 0, row - rowStart, 2);
                for (int i = 0; i < all_dependents.Count; i++)
                {
                    if ((bool)all_dependents[i].HasFormula)
                    {
                        wks.Cells[rowStart + 1 + i, 1].ApplyStyle(BuiltInStyle.Calculation);
                        wks.Cells[rowStart + 1 + i, 1].Style.WrapText = true;
                        sv.SetRowHeight(rowStart + 1 + i, sv.GetPreferredCellSize(rowStart + 1 + i, 1).Height);
                    }
                }
            }
            if (all_precedents.Count > direct_precedents.Count)
            {
                row += 2;
                sv.Cells[row, 0].Text = "All Precedents";
                sv.Cells[row, 0].Note = "Precedents are cells which are directly referenced in the target cell formula, and all cells which are directly referenced in those cell formulas, and so on.";
                sv.Cells[row, 0].ColumnSpan = 2;
                wks.Cells[row, 0].ApplyStyle(BuiltInStyle.Percent40Accent6);
                wks.Cells[row, 0].ApplyStyle(BuiltInStyle.Title);
                wks.Rows[row].RowHeight = wks.Rows[0].RowHeight;
                row++;
                rowStart = row;
                sv.Cells[row, 0].Text = "Cell";
                sv.Cells[row, 1].Text = "Formula/Value";
                row++;
                for (int i = 0; i < all_precedents.Count; row++, i++)
                {
                    sv.Cells[row, 0].Text = all_precedents[i].Address(false, false, ReferenceStyle.A1, false, all_precedents[i]);
                    sv.Cells[row, 1].Text = (bool)all_precedents[i].HasFormula ? (all_precedents[i].HasArray ? all_precedents[i].FormulaArray : all_precedents[i].Formula) : all_precedents[i].Text;
                }
                sv.AddTable(rowStart, 0, row - rowStart, 2);
                for (int i = 0; i < all_precedents.Count; i++)
                {
                    if ((bool)all_precedents[i].HasFormula)
                    {
                        wks.Cells[rowStart + 1 + i, 1].ApplyStyle(BuiltInStyle.Calculation);
                        wks.Cells[rowStart + 1 + i, 1].Style.WrapText = true;
                        sv.SetRowHeight(rowStart + 1 + i, sv.GetPreferredCellSize(rowStart + 1 + i, 1).Height);
                    }
                }
            }
            wks.RowCount = sv.GetLastNonEmptyRow(NonEmptyItemFlag.Data) + 1;
            sv.SetColumnWidth(1, 500);
            sv.ColumnCount = 2;
            splitContainer3.Panel2Collapsed = false;
            fpSpread2.Visible = true;
            hideReportPaneToolStripMenuItem.Visible = true;
        }

        private void hideReportPaneToolStripMenuItem_Click(object sender, EventArgs e)
        {
            splitContainer3.Panel2Collapsed = true;
            hideReportPaneToolStripMenuItem.Visible = false;
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            linkLabel1.LinkVisited = true;
            System.Diagnostics.Process.Start("https://templates.office.com/en-us/Sudoku-puzzle-solver-TM10080972");
        }
    }
}
